/// Header files.
#include <windows.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include <Commctrl.h>
#include <Commdlg.h>
#include "resource.h"
#include "dsobject.h"
#include "handleDeclaration.h"

/// Declarations (static - scope between different procedures).
CONST DWORD originalWidth = 480;
CONST DWORD originalHeight = 320;
static handleStruct handleOwner;
static dsObjectStruct dsObject;
static state currentState = UNDEFINE;
static LONGLONG currentPosition = 0;
static LONGLONG stopPosition = 0;
static LONG volume = MAX_VOL;
static BOOL isMute = FALSE;
static BOOL isFullScreen = FALSE;

/// Procedures declarations.
LRESULT CALLBACK MainWinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK VideoWinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPWSTR lpCmdLine,
                   int nCmdShow)
{
	MSG message;
	BOOL retVal = FALSE;

	/// Initialize handle structure variable.
	HandleStructInitialize(&handleOwner);

	/// Design a window class that will be created by CreateWindow procedure.
	handleOwner.hCurrentInst = hInstance;
	handleOwner.mainWindow.cbWndExtra = sizeof(handleOwner.videoWindow);
	handleOwner.mainWindow.hbrBackground = (HBRUSH)(COLOR_APPWORKSPACE + 2);
	handleOwner.mainWindow.hCursor = LoadCursor(NULL, NULL);
	handleOwner.mainWindow.hInstance = handleOwner.hCurrentInst;
	handleOwner.mainWindow.lpfnWndProc = (WNDPROC)MainWinProc;
	handleOwner.mainWindow.lpszClassName = L"Main Window";
	handleOwner.mainWindow.style = CS_VREDRAW | CS_HREDRAW;

	/// Register the class that will be created by CreateWindow procedure.
	if ((handleOwner.regMainWindow = RegisterClass(&(handleOwner.mainWindow))) == 0)
	{
		MessageBox(NULL, L"Main window registration failed.", L"Failure", MB_ICONERROR);
		return 0;
	}

	/// Create main window.
	handleOwner.hMainWindow = CreateWindow(L"Main Window", L"Video Playback in VC++", WS_DLGFRAME | WS_OVERLAPPED | WS_MINIMIZEBOX | WS_SYSMENU | WS_VISIBLE, 0, 0, 520, 430, NULL, NULL, hInstance, NULL);
	if (handleOwner.hMainWindow == NULL)
	{
		MessageBox(NULL, L"Main window creation failed.", L"Failure", MB_ICONERROR);
		return 0;
	}

	/// Initialize COM object and query required interfaces.
	retVal = DsObjectInitialize(&dsObject);
	if (retVal == FALSE)
	{
		MessageBox(NULL, L"COM object failed to initialize.", L"Error", MB_ICONERROR);
		return 0;
	}

	ZeroMemory(&message, sizeof(MSG));
	/// Loop will continue until WM_QUIT received from WinProc function.
	while(GetMessage(&message, NULL, 0, 0))
	{
		TranslateMessage(&message);
		DispatchMessage(&message);
	}
	/// Clean up.
	HandleStructCleanup(&handleOwner);
	DsObjectCleanup(&dsObject);
	return 0;
}

/// Procedure for main window.
LRESULT CALLBACK MainWinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	BOOL retVal = FALSE;
	DWORD count = 0;
	HWND tempHandle = NULL;
	//printf("\nMain window = %x %x", message, wParam);
	switch(message)
	{
		case WM_CREATE:
		{
			/// Design a window class that will be created by CreateWindow procedure.
			handleOwner.videoWindow.hbrBackground = (HBRUSH)(COLOR_BACKGROUND);
			handleOwner.videoWindow.hCursor = LoadCursor(NULL, NULL);
			handleOwner.videoWindow.hInstance = handleOwner.hCurrentInst;
			handleOwner.videoWindow.lpfnWndProc = (WNDPROC)VideoWinProc;
			handleOwner.videoWindow.lpszClassName = L"Video Window";
			handleOwner.videoWindow.lpszMenuName = NULL;
			handleOwner.videoWindow.style = CS_VREDRAW | CS_HREDRAW | CS_PARENTDC;

			/// Register the class that will be created by CreateWindow procedure.
			if ((handleOwner.regVideoWindow = RegisterClass(&(handleOwner.videoWindow))) == 0)
			{
				MessageBox(NULL, L"Sub window registration failed.", L"Failure", MB_ICONERROR);
				return 0;
			}
			
			/// Create a small window on main window to stream video.
			handleOwner.hVideoWindow = CreateWindow(L"Video Window", NULL, WS_CHILD | WS_VISIBLE, 20, 40, originalWidth, originalHeight, hWnd, NULL, handleOwner.hCurrentInst, NULL);
			if (hWnd == NULL)
			{
				MessageBox(NULL, L"Sub window creation failed.", L"Failure", MB_ICONERROR);
				return 0;
			}

			/// Draw command bar on main window.
			handleOwner.hCommandBar = CommandBar_Create(handleOwner.hCurrentInst, hWnd, ID_CMDBAR);
			/// Insert menu bar from resource file to command bar.
			CommandBar_InsertMenubar(handleOwner.hCommandBar, handleOwner.hCurrentInst, IDR_MENU1, 0); 
			/// Get handle to menu bar drawn on command bar.
			handleOwner.hMenuBar = CommandBar_GetMenu(handleOwner.hCommandBar, 0);

			/// Set parameters of dialog box to open for file selection.
			handleOwner.openDialogStruct.lStructSize = sizeof(handleOwner.openDialogStruct);
			handleOwner.openDialogStruct.hwndOwner = hWnd;
			handleOwner.openDialogStruct.lpstrFilter = L"Common media files\0*.MP3;*.WAV;*.OGG;*.WMA;*.AVI;*.MP4;*.MPEG;*.WMV;*.MOV;*.DIVX\0All files\0*.*\0\0";
			handleOwner.openDialogStruct.lpstrFile = handleOwner.filePathAndName;
			handleOwner.openDialogStruct.lpstrFileTitle = handleOwner.fileName;
			handleOwner.openDialogStruct.nMaxFile = BUFFER / sizeof(WCHAR);
			handleOwner.openDialogStruct.nMaxFileTitle = BUFFER / sizeof(WCHAR);
			handleOwner.openDialogStruct.Flags = OFN_PATHMUSTEXIST;

			/// Create buttons on main window.
			CreateWindow(L"BUTTON", L"OPEN", WS_VISIBLE | WS_CHILD, 20, 370, 115, 25, hWnd, (HMENU)ID_OPEN_BUTTON, NULL, NULL);
			CreateWindow(L"BUTTON", L"PLAY", WS_VISIBLE | WS_CHILD, 142, 370, 115, 25, hWnd, (HMENU)ID_PLAY_BUTTON, NULL, NULL);
			CreateWindow(L"BUTTON", L"PAUSE", WS_VISIBLE | WS_CHILD, 264, 370, 115, 25, hWnd, (HMENU)ID_PAUSE_BUTTON, NULL, NULL);
			CreateWindow(L"BUTTON", L"STOP", WS_VISIBLE | WS_CHILD, 386, 370, 115, 25, hWnd, (HMENU)ID_STOP_BUTTON, NULL, NULL);
			break;
		}
		case WM_COMMAND:
		{
			switch(wParam)
			{
				case ID_OPEN_BUTTON:
				case ID_FILE_LOAD:
					{
						/// Change focus from open button to main window.
						SetFocus(handleOwner.hMainWindow);
						ZeroMemory(&(handleOwner.filePathAndName), BUFFER);
						ZeroMemory(&(handleOwner.fileName), BUFFER);
						/// Get file name selected by user.
						GetOpenFileName(&(handleOwner.openDialogStruct));
						if (!wcscmp(handleOwner.filePathAndName, L""))
						{
							break;
						}
						if (currentState != UNDEFINE)
						{
							Reset(&dsObject);
							currentState = UNDEFINE;
						}
						Sleep(100);
						/// Start rendering on selected file.
						dsObject.pGraph->RenderFile(handleOwner.filePathAndName, NULL);
						EnableMenuItem(handleOwner.hMenuBar, ID_VIDEO_FULLSCREEN, MF_GRAYED);
						currentState = LOAD;
						break;
					}
				case ID_PLAY_BUTTON:
					{
						/// Change focus from play button to main window.
						SetFocus(handleOwner.hMainWindow);
						if (currentState == UNDEFINE)
						{
							break;
						}
						else if (currentState == LOAD)
						{
							/// Make video window as owner for streaming video.
							dsObject.pWindow->put_Owner((LONG)handleOwner.hVideoWindow);
							/// Dimensions of video to set.
							dsObject.pWindow->SetWindowPosition(0, 0, originalWidth, originalHeight);
							dsObject.pWindow->put_Visible(OATRUE);
							Sleep(50);
							/// Drain messages of video window into main window.
							dsObject.pWindow->put_MessageDrain((LONG)handleOwner.hMainWindow);
						}
						else if (currentState == PLAY)
						{
							/// Get current position.
							dsObject.pSeek->GetPositions(&currentPosition, &stopPosition);
							/// If no more data left to stream, stop all the filters.
							if (currentPosition >= stopPosition)
							{
								/// Stop all the filters in filter graph.
								dsObject.pMediaControl->Stop();
								Sleep(50);
								/// Set video streaming from starting.
								dsObject.pSeek->SetPositions(&(dsObject.startDuration), AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
								currentState = STOP;
								Sleep(50);
							}
						}
						if (currentState != PLAY)
						{
							/// Run filter graph.
							dsObject.pMediaControl->Run();
							if (isMute == TRUE)
							{
								/// Set volume.
								dsObject.pAudio->put_Volume(MIN_VOL);
							}
							else
							{
								dsObject.pAudio->put_Volume(volume);
							}
						}
						EnableMenuItem(handleOwner.hMenuBar, ID_VIDEO_FULLSCREEN, MF_ENABLED);
						currentState = PLAY;
						break;
					}
				case ID_PAUSE_BUTTON:
					{
						if (currentState == PLAY)
						{
							/// Pause all the filters.
							dsObject.pMediaControl->Pause();
							currentState = PAUSE;
						}
						/// Change focus from pause button to main window.
						SetFocus(handleOwner.hMainWindow);
						EnableMenuItem(handleOwner.hMenuBar, ID_VIDEO_FULLSCREEN, MF_ENABLED);
						break;
					}
				case ID_STOP_BUTTON:
					{
						if (currentState == PLAY || currentState == PAUSE)
						{
							/// Stop all the filters.
							dsObject.pMediaControl->Stop();
							Sleep(50);
							/// Set video streaming from starting.
							dsObject.pSeek->SetPositions(&(dsObject.startDuration), AM_SEEKING_AbsolutePositioning, NULL, AM_SEEKING_NoPositioning);
							currentState = STOP;
						}
						/// Change focus from stop button to main window.
						SetFocus(handleOwner.hMainWindow);
						EnableMenuItem(handleOwner.hMenuBar, ID_VIDEO_FULLSCREEN, MF_GRAYED);
						break;
					}
				case ID_FILE_QUIT:
					{
						/// Send quit message to close the application.
						PostQuitMessage(0);
						break;
					}
				case ID_VOLUME_UP:
					{
						isMute = FALSE;
						EnableMenuItem(handleOwner.hMenuBar, ID_VOLUME_DOWN, MF_ENABLED);
						CheckMenuItem(handleOwner.hMenuBar, ID_VOLUME_MUTE, MF_UNCHECKED);
						volume += 250;
						if (volume >= MAX_VOL)
						{
							volume = MAX_VOL;
							EnableMenuItem(handleOwner.hMenuBar, ID_VOLUME_UP, MF_GRAYED);
						}
						if (currentState == PLAY)
					    {
							/// Set volume.
							dsObject.pAudio->put_Volume(volume);
						}
						break;
					}
				case ID_VOLUME_DOWN:
					{
						isMute = FALSE;
						EnableMenuItem(handleOwner.hMenuBar, ID_VOLUME_UP, MF_ENABLED);
						CheckMenuItem(handleOwner.hMenuBar, ID_VOLUME_MUTE, MF_UNCHECKED);
						volume -= 250;
						if (volume <= MIN_VOL)
						{
							volume = MIN_VOL;
							EnableMenuItem(handleOwner.hMenuBar, ID_VOLUME_DOWN, MF_GRAYED);
						}
						if (currentState == PLAY)
					    {
							/// Set volume.
							dsObject.pAudio->put_Volume(volume);
						}
						break;
					}
				case ID_VOLUME_MUTE:
					{
						if (isMute == TRUE)
						{
							CheckMenuItem(handleOwner.hMenuBar, ID_VOLUME_MUTE, MF_UNCHECKED);
							if (currentState == PLAY)
							{
								/// Set volume.
								dsObject.pAudio->put_Volume(volume);
							}
							isMute = FALSE;
						}
						else
						{
							CheckMenuItem(handleOwner.hMenuBar, ID_VOLUME_MUTE, MF_CHECKED);
							if (currentState == PLAY)
							{
								/// Set volume.
								dsObject.pAudio->put_Volume(MIN_VOL);
							}
							isMute = TRUE;
						}
						break;
					}
				case ID_VIDEO_FULLSCREEN:
					{
						if (isFullScreen == FALSE && (currentState == PLAY || currentState == PAUSE))
						{
							/// Make video dimensions fit to maximum size of active display.
							dsObject.pWindow->put_FullScreenMode(OATRUE);
							Sleep(50);
							isFullScreen = TRUE;
						}
						break;
					}
			}
			break;
		}
		case WM_CHAR:
		case WM_LBUTTONUP:
			{
				/// Process Escape (ASCII = 27) key pressed on keyboard or mouse click to escape from full screen mode to normal mode.
				if (((message == WM_CHAR && wParam == ESC_KEY_ASCII) || message == WM_LBUTTONUP) && isFullScreen == TRUE)
				{
					dsObject.pWindow->put_FullScreenMode(OAFALSE);
					dsObject.pWindow->SetWindowPosition(0, 0, originalWidth, originalHeight);
					Sleep(50);
					isFullScreen = FALSE;
					/// Set focus to main window.
					SetFocus(handleOwner.hMainWindow);
				}
				break;
			}
		case WM_DESTROY:
		{
			/// Send quit message to close the application.
			PostQuitMessage(0);
			break;
		}
		default:
		{
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
	}
	/// To suppress the warning.
	return 0;
}

/// Procedure for video/sub window.
LRESULT CALLBACK VideoWinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	//printf("\nVideo window = %x %x", message, wParam);
	return DefWindowProc (hWnd, message, wParam, lParam);
}